import xml.etree.ElementTree
import numpy
import cv2
import random
import pickle
import os

#Convert RGB AllLines.png into BlackandWhite AllLines.png
def fConvertImageRGB2RGBBW(iImage):
    oImage=iImage.copy()
    vImageWidth=oImage.shape[1]
    vImageHeight=oImage.shape[0]
    vBlackColor=numpy.array([0,0,0])
    for vRowIndex in range(vImageWidth):
        for vColumnIndex in range(vImageHeight):
            vPixelColor=oImage[vColumnIndex,vRowIndex]
            if(numpy.array_equal(vPixelColor,vBlackColor)==False):#Line
                oImage[vColumnIndex,vRowIndex]=[255,255,255]
    return oImage
#Goes Over All Pixels of AllLinesPNG
#Apply "FloodFill" to all Black Pixes that it encounters
#Extracts and saves colored region to AllRegions folder.
def fGetRegionsFromAllLinesPNG(iImage,iFileName):
    oAllRegionsImage=iImage.copy()
    vRegionColors=[[0,0,0],[255,255,255]]
    vImageWidth=oAllRegionsImage.shape[1]
    vImageHeight=oAllRegionsImage.shape[0]
    vBlackColor=numpy.array([0,0,0])
    oRegionImages=[]
    for vRowIndex in range(vImageWidth):
        for vColumnIndex in range(vImageHeight):
            vPixelColor=oAllRegionsImage[vColumnIndex,vRowIndex]
            if(numpy.array_equal(vPixelColor,vBlackColor)==True):#NewRegion
                vRegionColor=[0,0,0]
                while(True):
                    vRColor=random.randint(40,255)
                    vGColor=random.randint(40,255)
                    vBColor=random.randint(40,255)
                    vRegionColor=[vRColor,vGColor,vBColor]
                    if(vRegionColor not in vRegionColors):
                        vRegionColors.append(vRegionColor)#Paints Region with a random color
                        break
                vFloodFillMask=numpy.zeros((vImageHeight+2,vImageWidth+2),numpy.uint8)
                cv2.floodFill(oAllRegionsImage,vFloodFillMask,(vRowIndex,vColumnIndex),vRegionColor)
                vRegionImage=numpy.zeros((vImageHeight,vImageWidth,3),numpy.uint8)
                vColoredPixels=oAllRegionsImage==vRegionColor
                vRegionImage[numpy.where(vColoredPixels.all(axis=2))]=vRegionColor
                oRegionImages.append(vRegionImage)
    cv2.imwrite(iFileName+"_Data/AllRegions.png",oAllRegionsImage)#Saves AllRegions into AllRegions.png
    return oRegionImages
#Parses the SVG and extracts all complete lines
#Each Complete Line is either a vanishing cycle or the foot of a handle
def fParseSVG(iFileName):
    oVanishingCycleLabelDict={}
    oHandleNamingDict={}
    vXMLTree=xml.etree.ElementTree.parse(iFileName)
    vRoot=vXMLTree.getroot()
    vImageWidth=int(vRoot.attrib["width"].split(".")[0])+1
    vImageHeight=int(vRoot.attrib["height"].split(".")[0])+1
    vPaths=vRoot.findall("{http://www.w3.org/2000/svg}g")[0]
    vPathsTranslateValues=[vPaths.attrib["transform"].split(",")[0].split("(")[1],vPaths.attrib["transform"].split(",")[1].split(")")[0]]
    vPathsTranslateValues=[float(vPathsTranslateValues[0]),float(vPathsTranslateValues[1])]
    vScaling=2#Scale the Image to avoid single pixel Regions. Each Region needs to be bigger than single pixel
    vCompleteLines=[]
    vCurrentPathIndex=0
    fMakeDir(iFileName+"_Data/CompleteLineImages/")
    for vPath in vPaths:#Extract all Path information on SVG. Complete Lines appear as Paths in SVG
        if(vPath.tag!="{http://www.w3.org/2000/svg}path"):
            continue
        vPathIndex=vCurrentPathIndex
        vCurrentPathIndex=vCurrentPathIndex+1
        vDescFound=False
        if(len(vPath)>0):
            for vChild in vPath:
                if("desc" in vChild.tag):
                    vDescFound=True
                    vDescription=vChild.text
                    if(vDescription[0]=='h'):#Path is a Handle Foot
                        vHandleID=100+vPathIndex
                        oVanishingCycleLabelDict[vPathIndex]=100+vPathIndex
                        oHandleNamingDict[vHandleID]=vDescription
                    else:#Path is a Vanishing cycle
                        vVanishingCycleName=vChild.text[1:]
                        oVanishingCycleLabelDict[vPathIndex]=int(vVanishingCycleName)
        if(vDescFound==False):#Path is a Handle Foot
            raise Exception("No Description Found")
        vPathDrawCommands=vPath.attrib["d"]
        vPathDrawCommands=vPathDrawCommands.split(" ")
        vCurrentX=0
        vCurrentY=0
        vCurrentMode=0
        vCurrentPathStartX=0
        vCurrentPathStartY=0
        vLinePoints=[]
        for vCommand in vPathDrawCommands:
            vValue1=0
            vValue2=0
            vValue=0
            if(vCommand.isalpha()==True):
                if(vCommand in ["m","M","h","H","v","V","z","Z","L","l"]):
                    vCurrentMode=vCommand
                else:
                    raise Exception("Error:SVGUnsupportedPath "+vCommand)
                if(vCurrentMode=="m" or vCurrentMode=="M"):
                    vLinePoints.append([])
                elif(vCurrentMode=="Z" or vCurrentMode=="z"):
                    vCurrentX=vCurrentPathStartX
                    vCurrentY=vCurrentPathStartY
                    vLinePoints.append([vCurrentX,vCurrentY])
            else:
                if("," in vCommand):
                    vValue1=float(vCommand.split(",")[0])
                    vValue2=float(vCommand.split(",")[1])
                else:
                    vValue=float(vCommand)
                if(vCurrentMode=="M"):
                    vCurrentX=vValue1
                    vCurrentY=vValue2
                    vCurrentPathStartX=vCurrentX
                    vCurrentPathStartY=vCurrentY
                elif(vCurrentMode=="m"):
                    vCurrentX=vValue1+vCurrentX
                    vCurrentY=vValue2+vCurrentY
                    vCurrentPathStartX=vCurrentX
                    vCurrentPathStartY=vCurrentY
                elif(vCurrentMode=="H"):
                    vCurrentX=vValue
                elif(vCurrentMode=="h"):
                    vCurrentX=vCurrentX+vValue
                elif(vCurrentMode=="V"):
                    vCurrentY=vValue
                elif(vCurrentMode=="v"):
                    vCurrentY=vCurrentY+vValue
                elif(vCurrentMode=="L"):
                    vCurrentX=vValue1
                    vCurrentY=vValue2
                elif(vCurrentMode=="l"):
                    vCurrentX=vValue1+vCurrentX
                    vCurrentY=vValue2+vCurrentY
                vLinePoints.append([vCurrentX,vCurrentY])
        vLinePointsTranslated=[]
        for vLinePoint in vLinePoints:
            if(vLinePoint==[]):
                vNewLinePoint=[]
            else:
                vNewLinePoint=[vLinePoint[0]+vPathsTranslateValues[0],vLinePoint[1]+vPathsTranslateValues[1]]
            vLinePointsTranslated.append(vNewLinePoint)
        vCompleteLines.append(vLinePointsTranslated)
    vAllLinesImage=numpy.zeros((vImageHeight*vScaling,vImageWidth*vScaling,3),numpy.uint8)
    oAllLineImages=[]#Drawings of Complete Lines
    for vCompleteLineIndex in range(len(vCompleteLines)):
        vLineImage=numpy.zeros(vAllLinesImage.shape)
        vLinePointsTranslated=vCompleteLines[vCompleteLineIndex]
        vLineStart=vLinePointsTranslated[1]
        vRColor=random.randint(40,255)#Sets a Random Color for the Complete Line Image
        vGColor=random.randint(40,255)
        vBColor=random.randint(40,255)
        for vLineEndIndex in range(2,len(vLinePointsTranslated)):
            vLineEnd=vLinePointsTranslated[vLineEndIndex]
            if(vLineEnd==[]):
                if(vLineEndIndex==1):
                    continue
                vLineEndIndex=vLineEndIndex+1
                vLineStart=vLinePointsTranslated[vLineEndIndex]
                vLineEndIndex=vLineEndIndex+1
                vLineEnd=vLinePointsTranslated[vLineEndIndex]
            vLineStartScaled=[]
            vLineStartScaled.append(int(int(vLineStart[0])*vScaling))
            vLineStartScaled.append(int(int(vLineStart[1])*vScaling))
            vLineEndScaled=[]
            vLineEndScaled.append(int(int(vLineEnd[0])*vScaling))
            vLineEndScaled.append(int(int(vLineEnd[1])*vScaling))
            vLineImage=cv2.line(vLineImage,tuple(vLineStartScaled),tuple(vLineEndScaled),(vRColor,vGColor,vBColor),1)
            vAllLinesImage=cv2.line(vAllLinesImage,tuple(vLineStartScaled),tuple(vLineEndScaled),(vRColor,vGColor,vBColor),1)
            vLineStart=vLineEnd
        cv2.imwrite(iFileName+"_Data/CompleteLineImages/"+str(len(oAllLineImages))+".png",vLineImage)#Saves Complete Line Images into folder CompleteLineImages
        oAllLineImages.append(vLineImage)
    cv2.imwrite(iFileName+"_Data/AllLines.png",vAllLinesImage)#Saves SVG Image
    return oAllLineImages,oVanishingCycleLabelDict,oHandleNamingDict

#Reads AllLines.png (generated by the program at the end of ParseSVG function)and Extracts Region Information.
def fParseRegions(iFileName):
    vAllLinesImage=cv2.imread(iFileName+"_Data/AllLines.png")
    vAllLinesImage=fConvertImageRGB2RGBBW(vAllLinesImage)
    oRegionImages=fGetRegionsFromAllLinesPNG(vAllLinesImage,iFileName)
    return oRegionImages

def fMakeDir(iPath):
    if not os.path.exists(iPath):
        os.makedirs(iPath)
#Extracts Corner and Boundry Arc information from each region
#Also Saves Region with its relevant complete lines as a PNG in RegionImagesLines
def fGetRegionBoundryData(iAllLineImages,iAllRegionImages,iFileName):
    def fGetTopLeftCornerRegion(iRegionImage):
        for vY in range(iRegionImage.shape[0]):
            for vX in range(iRegionImage.shape[1]):
                vPixelColor=iRegionImage[vY,vX,:]
                if(numpy.array_equal(vPixelColor,[0,0,0])==False):
                    return [vX,vY]
        print("Error:fGetTopLeftCornerRegion")
    def fGetTopLeftCornerRegion0(iRegionImage):
        for vY in range(iRegionImage.shape[0]):
            for vX in range(iRegionImage.shape[1]):
                vPixelColor=iRegionImage[vY,vX,:]
                if(numpy.array_equal(vPixelColor,[0,0,0])==True):
                    return [vX,vY]
        print("Error:fGetTopLeftCornerRegion")
    #Finds which complete lines are present in the given pixel location
    def fGetLineIDFromLocation(iLocation,iAllLineImages):
        oLineIDs=[]
        for vLineImageIndex in range(len(iAllLineImages)):
            vLineImage=iAllLineImages[vLineImageIndex]
            vPixelColor=vLineImage[iLocation[1],iLocation[0]]
            if(not numpy.array_equal(vPixelColor,[0,0,0])):
                oLineIDs.append(vLineImageIndex)
        if(len(oLineIDs)>0):
            return oLineIDs
        return -1
    #Checks Crossings with Handle Feet
    def fCheckFakeCrossingLocation(iLocation,iPreviousLocation,iImage):#OnlyHappensinC100+s
        for vTestChange in [[0,-1],[0,1],[-1,0],[1,0]]:
            vTestLocation=[iLocation[0]+vTestChange[0],iLocation[1]+vTestChange[1]]
            if(not numpy.array_equal(iImage[vTestLocation[1],vTestLocation[0],:],[0,0,0])):
                if(vTestLocation!=iPreviousLocation):
                    return True
        return False
    def fGetNewDirectionCorner(iLocation,iPreviousLocation,iImage):
        vSelectedChange=-1
        for vTestChange in [[-1,-1],[1,-1],[-1,1],[1,1]]:
            vTestLocation=[iLocation[0]+vTestChange[0],iLocation[1]+vTestChange[1]]
            if(not numpy.array_equal(iImage[vTestLocation[1],vTestLocation[0],:],[0,0,0])):
                if(iLocation[1]==iPreviousLocation[1]):
                    return [0,vTestChange[1]]
                elif(iLocation[0]==iPreviousLocation[0]):
                    return [vTestChange[0],0]
        raise Exception("Error:vSelectedChange=-1")
    #Selects Next pixel on the Boundry
    def fGetNewDirectionLine(iLocation,iPreviousLocation,iImage):
        for vTestChange in [[0,-1],[0,1],[-1,0],[1,0]]:
            vTestLocation=[iLocation[0]+vTestChange[0],iLocation[1]+vTestChange[1]]
            if(not numpy.array_equal(iImage[vTestLocation[1],vTestLocation[0],:],[0,0,0])):
                if(vTestLocation!=iPreviousLocation):
                    return vTestChange
        raise Exception("Non Vertical/Horizontal Line Detected")
    oRegionInformation=[]
    vRegionImageLocation=iFileName+"_Data/RegionImages/"
    vRegionImageWithLinesLocation=iFileName+"_Data/RegionImagesLines/"
    fMakeDir(vRegionImageLocation)
    fMakeDir(vRegionImageWithLinesLocation)
    for vRegionImageIndex in range(0,len(iAllRegionImages)):#len(iAllRegionImages)
        print(vRegionImageIndex+1,"/",len(iAllRegionImages))#Prints the Progress
        vCrossingLocations=[]
        vLines=[]
        vRegionImage=iAllRegionImages[vRegionImageIndex]
        vDisplayImage=vRegionImage.copy()
        cv2.imwrite(vRegionImageLocation+str(vRegionImageIndex)+".png",vDisplayImage)
        vTopLeftCorner=fGetTopLeftCornerRegion(vRegionImage)#Finds Leftmost of the top corners Of the Region called first corner below
        if(vRegionImageIndex==0):#RegionImageIndex=0 is the outermost region. 
            vTopLeftCorner=fGetTopLeftCornerRegion0(vRegionImage)#Finds Leftmost of the top corners of the compliment of Region 0
            vStartLocation=[vTopLeftCorner[0],vTopLeftCorner[1]]
        else:
            vStartLocation=[vTopLeftCorner[0]-1,vTopLeftCorner[1]]#Finds the crossing at the first corner 
        #Selected pixel proceeds counter-clockwise along the Boundary of the region, keeping track of all CompleteLines that contain the selected pixel. When the number of CompleteLines containing the selected pixel increases to 2, a crossing is detected along with the identifying data of its CompleteLines. 
        vStartLineID=fGetLineIDFromLocation(vStartLocation,iAllLineImages)
        if(vStartLineID==-1 or len(vStartLineID)!=1):
            raise Exception("First Pixel Not Detected")
        vStartLineID=vStartLineID[0]
        vLineImage=iAllLineImages[vStartLineID]
        vCurrentLocation=[vStartLocation[0],vStartLocation[1]+1]
        vPreviousLocation=vStartLocation#Keeps track of previous location to select other ComleteLines in corners
        vLines.append(vStartLineID)
        vDisplayImage=vDisplayImage+vLineImage
        vCurrentLineID=vStartLineID
        while(True):
            if(vCurrentLocation==vStartLocation):#Ends at first crossing
                break
            vTestLineIDs=fGetLineIDFromLocation(vCurrentLocation,iAllLineImages)
            if(vTestLineIDs==-1 or len(vTestLineIDs)>2):
                raise Exception("Pixel Not Detected or more than 2 CompleteLines present")
            if(len(vTestLineIDs)==2):#PossibleCrossing
                #Checks Crossings with Handle Feet
                vFakeChange=fCheckFakeCrossingLocation(vCurrentLocation,vPreviousLocation,vRegionImage)
                if(vFakeChange==False):#CrossingFound
                    vDisplayImage=cv2.circle(vDisplayImage,vCurrentLocation,3,(0,255,0),-1)
                    if(vTestLineIDs[0]==vCurrentLineID):
                        vCurrentLineID=vTestLineIDs[1]
                    else:
                        vCurrentLineID=vTestLineIDs[0]
                    #Start Following the Other Line
                    vLineImage=iAllLineImages[vCurrentLineID]
                    vLines.append(vCurrentLineID)
                    vCrossingLocations.append(vCurrentLocation)
                    vDisplayImage=vDisplayImage+vLineImage
                    vChange=fGetNewDirectionCorner(vCurrentLocation,vPreviousLocation,vRegionImage)
                    vPreviousLocation=vCurrentLocation
                    vCurrentLocation=[vCurrentLocation[0]+vChange[0],vCurrentLocation[1]+vChange[1]]
                    continue
            #Single CompleteLine Present in Pixel
            vDisplayImage=cv2.circle(vDisplayImage,vCurrentLocation,1,(0,0,255),-1)
            #cv2.imshow("RegionImage",vDisplayImage)#If you are interested in an animation showing boundary data detection uncomment this line and the next warning:slow
            #cv2.waitKey(1)
            #SelectNewLinePoint
            vChange=fGetNewDirectionLine(vCurrentLocation,vPreviousLocation,vLineImage)#Selects Next Pixel
            vPreviousLocation=vCurrentLocation
            vCurrentLocation=[vCurrentLocation[0]+vChange[0],vCurrentLocation[1]+vChange[1]]
        oRegionInformation.append([vCrossingLocations,vLines])
        cv2.imwrite(vRegionImageWithLinesLocation+str(vRegionImageIndex)+".png",vDisplayImage)
    return oRegionInformation
#Each Iteration correspons to a labeling of Vanishing Cycles. 
#Generates Label data of each Vanishing Cycles for each Iteration
def fGetIterationInformation(iLineTranslationDict,iFileName):
    def fGetChangableCycleIDs(iLineTranslationDict):
        oLineIDs=[]
        for vLineIndex in range(len(iLineTranslationDict)):
            vID=iLineTranslationDict[vLineIndex]
            if(vID<100):
                oLineIDs.append(vID)
        oLineIDs.sort()
        oLineIDs=list(numpy.unique(numpy.array(oLineIDs)))
        return oLineIDs
    def fGetModifiedLineTranslationDict(iOldID,iNewID,iLineTranslationDict):
        oVanishingCycleLabelDict=iLineTranslationDict.copy()
        for vLineIndex in range(len(iLineTranslationDict)):
            vID=oVanishingCycleLabelDict[vLineIndex]
            if(vID==iOldID):
                oVanishingCycleLabelDict[vLineIndex]=iNewID
        return oVanishingCycleLabelDict
    def fWriteIterationInformationToFile(iFileName,iChanges,iIterationInformation):
        vFile=open(iFileName,"w")
        vLine1="Line,Start"
        for vChange in iChanges:
            vLine1=vLine1+",c"+str(vChange[0])+"->c"+str(vChange[1])
        vLine1=vLine1+"\n"
        vFile.write(vLine1)
        for vLineIndex in range(len(iIterationInformation[0])):
            vLine=str(vLineIndex)
            for vIterationIndex in range(len(iIterationInformation)):
                vLine=vLine+",c"+str(iIterationInformation[vIterationIndex][vLineIndex])
            vLine=vLine+"\n"
            vFile.write(vLine)
        vFile.close()
    vChangableCycleIDs=fGetChangableCycleIDs(iLineTranslationDict)
    vChangableCycleLength=len(vChangableCycleIDs)
    vCurrentLineTranslationDict=iLineTranslationDict.copy()
    oIterationInformation=[]
    oIterationInformation.append(vCurrentLineTranslationDict)
    vChanges=[]
    for vIndex in range(vChangableCycleLength):
        vOldID=vChangableCycleIDs[0]
        vNewID=vOldID+vChangableCycleLength+1
        vCurrentLineTranslationDict=fGetModifiedLineTranslationDict(vOldID,vNewID,vCurrentLineTranslationDict)
        oIterationInformation.append(vCurrentLineTranslationDict)
        vChanges.append([vOldID,vNewID])
        vChangableCycleIDs.pop(0)
    fWriteIterationInformationToFile(iFileName,vChanges,oIterationInformation)
    return oIterationInformation
#Get Corner and CompleteLine information for each Region and updates them for each Iteration
def fGetCornerHVInformation(iRegionInformation,iIterationInformation,iAllLineImages,iFileName):
    def fGetCornersByRegion(iAllRegionInformation):
        oCorners={}
        for vRegionIndex in range(len(iAllRegionInformation)):
            vRegionInformation=iAllRegionInformation[vRegionIndex]
            vCorners=vRegionInformation[0]
            vLines=vRegionInformation[1]
            for vCornerIndex in range(len(vCorners)):
                vCorner=vCorners[vCornerIndex]
                vLine1=vLines[vCornerIndex]
                vLine2=vLines[vCornerIndex+1]
                vCornerName=str(vCorner[0])+"_"+str(vCorner[1])
                if(vCornerName not in oCorners):
                    oCorners[vCornerName]={}
                oCorners[vCornerName][vRegionIndex]=[vLine1,vLine2]
        #for vCorner in oCorners:
        #    print(vCorner,oCorners[vCorner])
        return oCorners
    def fGetCornerLines(iCornerNames,iAllLineImages):
        def fGetLineIDFromLocation(iX,iY,iAllLineImages):
            oLineIDs=[]
            for vLineImageIndex in range(len(iAllLineImages)):
                vLineImage=iAllLineImages[vLineImageIndex]
                vPixelColor=vLineImage[iY,iX]
                if(not numpy.array_equal(vPixelColor,[0,0,0])):
                    oLineIDs.append(vLineImageIndex)
            if(len(oLineIDs)==1):
                return oLineIDs[0]
            elif(len(oLineIDs)==0):
                return -1
            raise Exception("Error:fGetCornerLines/fGetLineIDFromLocation",oLineIDs)
        oCornerLines={}
        for vCornerName in iCornerNames:
            vX=int(vCornerName.split("_")[0])
            vY=int(vCornerName.split("_")[1])
            vLeftLine=fGetLineIDFromLocation(vX-1,vY,iAllLineImages)
            if(vLeftLine==-1):
                vLeftLine=fGetLineIDFromLocation(vX+1,vY,iAllLineImages)
            if(vLeftLine==-1):
                raise Exception("Error:fGetCornerLines/fGetLineIDFromLocation=-1Twice")
            vTopLine=fGetLineIDFromLocation(vX,vY-1,iAllLineImages)
            if(vTopLine==-1):
                vTopLine=fGetLineIDFromLocation(vX,vY+1,iAllLineImages)
            if(vTopLine==-1):
                raise Exception("Error:fGetCornerLines/fGetLineIDFromLocation=-1Twice")
            oCornerLines[vCornerName]=[vLeftLine,vTopLine]
        #for oCornerLine in oCornerLines:
        #    print(oCornerLine,oCornerLines[oCornerLine])
        return oCornerLines
    def fGetCornerHVByIteration(iCornerLines,iIterationInformation):
        oCornerHVByIteration=[]
        for vTranslationDict in iIterationInformation:
            vCornerData={}
            for vCorner in iCornerLines:
                vLeftLine=iCornerLines[vCorner][0]
                vTopLine=iCornerLines[vCorner][1]
                vLeftLine=vTranslationDict[vLeftLine]
                vTopLine=vTranslationDict[vTopLine]
                vLineOrder="v"
                if(vLeftLine>vTopLine):
                    vLineOrder="h"
                vCornerData[vCorner]=vLineOrder
            oCornerHVByIteration.append(vCornerData)
        return oCornerHVByIteration
    def fWriteCornerHVInformationToFile(iFileName,iCornerHVByIteration,iCornerLines):
        vFile=open(iFileName,"w")
        for vCornerName in iCornerHVByIteration[0]:
            vLine=vCornerName
            vLine=vLine+","+str(iCornerLines[vCornerName][0])+","+str(iCornerLines[vCornerName][1])
            for vIterationIndex in range(len(iCornerHVByIteration)):
                vLine=vLine+","+iCornerHVByIteration[vIterationIndex][vCornerName]
            vLine=vLine+"\n"
            vFile.write(vLine)
        vFile.close()
    vCorners=fGetCornersByRegion(iRegionInformation)
    vCornerLines=fGetCornerLines(vCorners.keys(),iAllLineImages)
    oCornerHVByIteration=fGetCornerHVByIteration(vCornerLines,iIterationInformation)
    fWriteCornerHVInformationToFile(iFileName,oCornerHVByIteration,vCornerLines)
    return oCornerHVByIteration
#Writes Equations into Equations.txt file
def fGenerateEquationInformation(iRegionInformation,iIterationInformation,iCornerHVInformation,iFileName,iAllCombinedRegionInformation):
    def fWriteDetailEquationInformation(iEquationInformationByIteration,iOrderInformationByIteration,iFileName):
        vFile=open(iFileName,"w")
        for vRegion in iEquationInformationByIteration[0]:
            vFile.write("||||||||||||||||||Region: "+str(vRegion)+"\n")
            for vIterationDictIndex in range(len(iEquationInformationByIteration)):
                vFile.write("------Iteration: "+str(vIterationDictIndex)+"\n")
                vFile.write(str(iOrderInformationByIteration[vIterationDictIndex][vRegion])+"\n")
                vFile.write(iEquationInformationByIteration[vIterationDictIndex][vRegion]+"\n")
        vFile.close()
    def fWriteEquations(iAllEquations,iFileName):
        vFile=open(iFileName,"w")
        vFile.write("{")
        for vEquationIndex in range(len(iAllEquations)-1):
            vEquation=iAllEquations[vEquationIndex]
            vFile.write(vEquation+",\n")
        vEquation=iAllEquations[-1]
        vFile.write(vEquation+"}")
        vFile.close()
    vAllEquations=[]
    vEquationInformationByIteration=[]
    vOrderInformationByIteration=[]
    for vIterationDictIndex in range(len(iIterationInformation)):
        vIterationDict=iIterationInformation[vIterationDictIndex]
        vIterationEquations={}
        vIterationOrderInformation={}
        for vRegionIndex in range(len(iRegionInformation)):
            vRegion=iRegionInformation[vRegionIndex]
            vLineChangesIteration=[]
            for vLine in vRegion[1]:
                vLineChangesIteration.append(vIterationDict[vLine])
            vLineChangesIterationString=[]
            for vLine in vLineChangesIteration:
                vLineChangesIterationString.append("c"+str(vLine))
            vSignsIteration=[]
            vNumberOfPlusSigns=0
            vNumberOfMinusSigns=0
            for vCornerIndex in range(len(vRegion[0])):
                vLine1=vLineChangesIteration[vCornerIndex]
                vLine2=vLineChangesIteration[vCornerIndex+1]
                vSign="-"
                if(vLine1<vLine2):
                    vSign="+"
                    vNumberOfPlusSigns=vNumberOfPlusSigns+1
                else:
                    vNumberOfMinusSigns=vNumberOfMinusSigns+1
                vSignsIteration.append(vSign)
            vEquation=""
            for vCornerIndex in range(len(vRegion[0])):
                vCornerLocation=vRegion[0][vCornerIndex]
                vSign=vSignsIteration[vCornerIndex]
                vCornerName=str(vCornerLocation[0])+"_"+str(vCornerLocation[1])
                vHVInformation=iCornerHVInformation[vIterationDictIndex][vCornerName]
                vParameterText=vSign+"c["+str(vCornerLocation[0])+","+str(vCornerLocation[1])+","+vHVInformation+"]"
                vEquation=vEquation+vParameterText
            if(vEquation!=""):
                if(vRegionIndex==0):
                    vEquation=vEquation+"==-1*("+str(vNumberOfPlusSigns)+"-"+str(vNumberOfMinusSigns)+")"
                else:
                    vEquation=vEquation+"=="+str(vNumberOfPlusSigns)+"-"+str(vNumberOfMinusSigns)
                if(vEquation not in vAllEquations):
                    vAllEquations.append(vEquation)
            vIterationEquations[vRegionIndex]=vEquation
            vIterationOrderInformation[vRegionIndex]=vLineChangesIterationString
        vEquationInformationByIteration.append(vIterationEquations)
        vOrderInformationByIteration.append(vIterationOrderInformation)
    vAllEquations=[]
    for vRegion in vEquationInformationByIteration[0]:
        vCombinedRegionType=0
        for vCombinedRegionInformation in iAllCombinedRegionInformation:
            vRegion1=vCombinedRegionInformation[0]
            vRegion2=vCombinedRegionInformation[1]
            if(vRegion1==vRegion):
                vCombinedRegionType=1
                break
            if(vRegion2==vRegion):
                vCombinedRegionType=2
                break
        for vIterationDictIndex in range(len(vEquationInformationByIteration)):
                if(vCombinedRegionType!=0):
                   if(vCombinedRegionType==1):
                       vEquation1=vEquationInformationByIteration[vIterationDictIndex][vRegion]
                       vEquation2=vEquationInformationByIteration[vIterationDictIndex][vRegion2]
                       vEquation=vEquation1.split("==")[0]+"+("+vEquation2.split("==")[0]+")=="+vEquation1.split("==")[1]+"+("+vEquation2.split("==")[1]+")"
                       for vCornerIndex in range(4):
                           vCornerLocation=vCombinedRegionInformation[2+vCornerIndex]
                           vCornerText1="c["+str(vCornerLocation[0])+","+str(vCornerLocation[1])+",h]"
                           vCornerText2="c["+str(vCornerLocation[0])+","+str(vCornerLocation[1])+",v]"
                           vEquation=vEquation.replace(vCornerText1,"0")
                           vEquation=vEquation.replace(vCornerText2,"0")
                           vEquation=vEquation.replace("+0","")
                           vEquation=vEquation.replace("-0","")
                           vEquation=vEquation.split("==")[0]
                           vNumberOfPlusSigns=vEquation.count('+')-1
                           vNumberOfMinusSigns=vEquation.count('-')
                           vEquation=vEquation+"=="+str(vNumberOfPlusSigns)+"-"+str(vNumberOfMinusSigns)
                       #vEquationInformationByIteration[vIterationDictIndex][vRegion]=vEquation
                   if(vCombinedRegionType==2):
                       continue
                else:
                    vEquation=vEquationInformationByIteration[vIterationDictIndex][vRegion]
                if(vEquation==""):
                    continue
                if(vEquation not in vAllEquations):
                    vAllEquations.append(vEquation)
    fWriteDetailEquationInformation(vEquationInformationByIteration,vOrderInformationByIteration,iFileName+"EquationInfo.txt")
    fWriteEquations(vAllEquations,iFileName+"Equations.txt")
    
def fGetHandleConnectionInformation(iHandleNamingDict,iAllLineImages,iLineTranslationDict,iAllRegionImages,iFileName):
        def fReverseLookup(iValue,iDictionary):
            oIndexes=[]
            for vKey,vValue in iDictionary.items():
                if(vValue==iValue):
                    oIndexes.append(vKey)
            return oIndexes
        def fGetTopLeftCornerLine(iLineImage):
            for vY in range(iLineImage.shape[0]):
                for vX in range(iLineImage.shape[1]):
                    vPixelColor=iLineImage[vY,vX,:]
                    if(numpy.array_equal(vPixelColor,[0,0,0])==False):
                        return [vX,vY]
        def fGetTopRightCornerLine(iLineImage):
            for vY in range(iLineImage.shape[0]):
                for vX in range(iLineImage.shape[1]-1,-1,-1):
                    vPixelColor=iLineImage[vY,vX,:]
                    if(numpy.array_equal(vPixelColor,[0,0,0])==False):
                        return [vX,vY]
        def fCheckIfInLine(iLocation,iLineImage):
            vPixelColor=iLineImage[iLocation[1],iLocation[0],:]
            if(numpy.array_equal(vPixelColor,[0,0,0])==False):
                        return True
            return False
        def fGetLineIDFromLocation(iLocation,iAllLineImages):
            oLineIDs=[]
            for vLineImageIndex in range(len(iAllLineImages)):
                vLineImage=iAllLineImages[vLineImageIndex]
                vPixelColor=vLineImage[iLocation[1],iLocation[0]]
                if(not numpy.array_equal(vPixelColor,[0,0,0])):
                    oLineIDs.append(vLineImageIndex)
            if(len(oLineIDs)>0):
                return oLineIDs
            return -1
        vHandleConnectionDict={}
        for vKey in iHandleNamingDict:
            vValue=iHandleNamingDict[vKey]
            vHandleNumber=int(vValue[1])
            if(vHandleNumber not in vHandleConnectionDict):
                vHandleConnectionDict[vHandleNumber]=[]
                vHandleConnectionDict[vHandleNumber].append([])
                vHandleConnectionDict[vHandleNumber].append([])
        for vKey in iHandleNamingDict:
            vLineID=fReverseLookup(vKey,iLineTranslationDict)[0]
            vHandleID=iHandleNamingDict[vKey]
            vLineImage=iAllLineImages[vLineID]
            vHandleOrder=vHandleID[2]
            vHandleNumber=int(vHandleID[1])
            vHandleOrderIndex=int(ord(vHandleOrder)-ord('a'))
            vStartLocation=0
            vDirection=[0,+1]
            if(vHandleOrder=='a'):
                vStartLocation=fGetTopLeftCornerLine(vLineImage)
            else:
                vStartLocation=fGetTopRightCornerLine(vLineImage)
            vCurrentLocation=vStartLocation
            while(True):
                vTestLineIDs=fGetLineIDFromLocation(vCurrentLocation,iAllLineImages)
                if(len(vTestLineIDs)==2):
                    vCrossLineID=vTestLineIDs[0]
                    if(vTestLineIDs[0]==vLineID):
                        vCrossLineID=vTestLineIDs[1]
                    vvRegionCheckPoint=[0,0]
                    if(vDirection==[0,+1]):
                        if(vHandleOrder=='a'):
                            vRegionCheckPoint=[vCurrentLocation[0]-1,vCurrentLocation[1]+1]
                        else:
                            vRegionCheckPoint=[vCurrentLocation[0]+1,vCurrentLocation[1]+1]
                    if(vDirection==[0,-1]):
                        if(vHandleOrder=='a'):
                            vRegionCheckPoint=[vCurrentLocation[0]+1,vCurrentLocation[1]-1]
                        else:
                            vRegionCheckPoint=[vCurrentLocation[0]-1,vCurrentLocation[1]-1]
                    if(vDirection==[+1,0]):
                        if(vHandleOrder=='a'):
                            vRegionCheckPoint=[vCurrentLocation[0]+1,vCurrentLocation[1]+1]
                        else:
                            vRegionCheckPoint=[vCurrentLocation[0]+1,vCurrentLocation[1]-1]
                    if(vDirection==[-1,0]):
                        if(vHandleOrder=='a'):
                            vRegionCheckPoint=[vCurrentLocation[0]-1,vCurrentLocation[1]-1]
                        else:
                            vRegionCheckPoint=[vCurrentLocation[0]-1,vCurrentLocation[1]+1]
                    vRegionID=fGetLineIDFromLocation(vRegionCheckPoint,iAllRegionImages)[0]
                    vCornerInformation=[vCurrentLocation[0],vCurrentLocation[1],vKey,vCrossLineID,vRegionID]
                    vHandleConnectionDict[vHandleNumber][vHandleOrderIndex].append(vCornerInformation)
                vTempLocation=[vCurrentLocation[0]+vDirection[0],vCurrentLocation[1]+vDirection[1]]
                if(fCheckIfInLine(vTempLocation,vLineImage)):
                    vCurrentLocation=vTempLocation
                else:
                    if(vDirection[0]==0):
                        vTempLocation=[vCurrentLocation[0]+1,vCurrentLocation[1]+0]
                        if(fCheckIfInLine(vTempLocation,vLineImage)):
                            vCurrentLocation=vTempLocation
                            vDirection=[1,0]
                        else:
                            vTempLocation=[vCurrentLocation[0]-1,vCurrentLocation[1]+0]
                            if(fCheckIfInLine(vTempLocation,vLineImage)):
                                vCurrentLocation=vTempLocation
                                vDirection=[-1,0]
                            else:
                                raise Exception("NonRectangleHandle LineID:",vLineID)
                    else: 
                        vTempLocation=[vCurrentLocation[0]+0,vCurrentLocation[1]-1]
                        if(fCheckIfInLine(vTempLocation,vLineImage)):
                            vCurrentLocation=vTempLocation
                            vDirection=[0,-1]
                        else:
                            raise Exception("NonRectangleHandle LineID:",vLineID)
                if(vCurrentLocation==vStartLocation):
                    break
        fMakeDir(iFileName+"_Data/CombinedRegionImages/")
        oAllCombinedRegionInformation=[]
        for vHandleId in vHandleConnectionDict:
            vConnectionData=vHandleConnectionDict[vHandleId]
            for vConnectionIndex in range(len(vConnectionData[0])):
                vRegion1=vConnectionData[0][vConnectionIndex][4]
                vRegion2=vConnectionData[1][vConnectionIndex][4]
                vCorner1=[vConnectionData[0][vConnectionIndex][0],vConnectionData[0][vConnectionIndex][1]]
                vCorner3=[vConnectionData[1][vConnectionIndex][0],vConnectionData[1][vConnectionIndex][1]]
                if(vConnectionIndex+1>=len(vConnectionData[0])):
                    vCorner2=[vConnectionData[0][0][0],vConnectionData[0][0][1]]
                    vCorner4=[vConnectionData[1][0][0],vConnectionData[1][0][1]]
                else:
                    vCorner2=[vConnectionData[0][vConnectionIndex+1][0],vConnectionData[0][vConnectionIndex+1][1]]
                    vCorner4=[vConnectionData[1][vConnectionIndex+1][0],vConnectionData[1][vConnectionIndex+1][1]]
                vCombinedRegionInformation=[vRegion1,vRegion2,vCorner1,vCorner2,vCorner3,vCorner4]
                oAllCombinedRegionInformation.append(vCombinedRegionInformation)
                vRegionImage1=iAllRegionImages[vRegion1]
                vRegionImage2=iAllRegionImages[vRegion2]
                vCombinedRegionImage=vRegionImage1+vRegionImage2
                cv2.imwrite(iFileName+"_Data/CombinedRegionImages/h"+str(vHandleId)+"_"+str(vConnectionIndex)+".png",vCombinedRegionImage)       
        return oAllCombinedRegionInformation
def fMain():#MainFunction
    vFileName="image.svg"
    vAllLineImages,vLineTranslationDict,vHandleNamingDict=fParseSVG(vFileName)
    vAllRegionImages=fParseRegions(vFileName)
    vRegionInformation=0
    if(True):
        vRegionBoundryData=fGetRegionBoundryData(vAllLineImages,vAllRegionImages,vFileName)
        pickle.dump(vRegionBoundryData,open(vFileName+"_Data/RegionData.pkl","wb"))#Saves Boundry Data
    else:
        vRegionBoundryData=pickle.load(open(vFileName+"_Data/RegionData.pkl","rb"))#Loads Boundry Data
    vAllCombinedRegionInformation=fGetHandleConnectionInformation(vHandleNamingDict,vAllLineImages,vLineTranslationDict,vAllRegionImages,vFileName)
    vIterationInformation=fGetIterationInformation(vLineTranslationDict,vFileName+"_Data/LineTranslation.csv")
    vCornerHVInformation=fGetCornerHVInformation(vRegionBoundryData,vIterationInformation,vAllLineImages,vFileName+"_Data/CornerHVInformation.csv")
    fGenerateEquationInformation(vRegionBoundryData,vIterationInformation,vCornerHVInformation,vFileName+"_Data/",vAllCombinedRegionInformation)
    
if __name__ == "__main__":
   fMain()#CallsMainFunction
