(*Description. This is the list of commands to produce three outputs: numbers10.txt, salientm10.txt and salientb10.txt. The file numbers10.txt has a list of numbers whose gcd is the modulus for which each salient sequence described by the other files comes from a valid choice of grading. In this case, that modulus is 0, so the grading system has integer solutions. The collection of salient sequences are of the form -salientm10.x-salientb10, where x ranges over Z^22, salientm10 is the matrix in salientm10.txt, and salientb10 is the vector in salientb10.txt.*)

(*Instructions. In Linux, feed this file to Mathematica with Equations10.txt in Mathematica's default directory using, say, "math < 10.m 2>&1 | tee 10.log". In Windows, open a notebook, type the command Directory[] and hit Shift+Enter. Put Equations10.txt into the folder outputted by Mathematica. Next, copy/paste this entire file into the notebook, and hit Shift+Enter. For Equations10.txt, it took a node of the Spiedie cluster with 92GB of RAM about 2 hours to complete. Machines with under 80GB of RAM are unlikely to be able to complete this calculation. To run your own equations, change the Import command to bring in Equations.txt instead of Equations10.txt, and change the export commands below to output whatever named files you like, other than numbers10.txt, and salientm10.txt, salientb10.txt.*)

(*Here begins the annotated sequence of Mathematica commands.*)

(*This is a command to delete all previously defined variables. It is commented out, You may want to un-comment it by deleting the (* *). The semicolon tells Mathematica to not print any output.*)
(*ClearAll["Global`*"];*)

(*Import the list of equations for $E(1)_{10_133}$. Call the resulting string eqstring.*)
eqstring = Import["Equations10.txt"];

(*Interpret eqstring as a list of equations instead of just a text string. Call the resulting list of equations eq*)
eq = ToExpression[eqstring];

(*List all of the variables in eq in the order in which they appear, including repeats. Call the resulting list of strings xstring.*)
xstring = StringCases[eqstring, RegularExpression["c[[0-9]+,[0-9]+,(h|v)]"]];

(*Interpret the entries of the list xstring as variables instead of just text strings. Call this list of variables x1.*)
x1 = ToExpression[xstring];

(*Call the list of salient chords s.*)
s = {c[1072, 332, v], c[1056, 316, v], c[1040, 302, v], 
   c[1024, 286, v], c[1008, 272, v], c[992, 256, v], c[976, 242, v], 
   c[960, 226, v], c[944, 212, v], c[928, 196, v], c[912, 182, v], 
   c[896, 166, v], c[880, 152, v], c[864, 136, v], c[848, 122, v], 
   c[832, 106, v], c[816, 92, v], c[800, 76, v], c[784, 62, v], 
   c[766, 46, v], c[766, 338, h], c[1090, 338, v]};

(*Insert the entries of s at the beginning of x1. Call this new list of variables x2.*)
x2 = Join[s, x1];

(*Delete the duplicates in x2. Call the resulting list of variables, which begins with the list of salient chords, x.*)
x = DeleteDuplicates[x2];

(*Create the coefficient array for the equations' left-hand sides and call it m. Create the row vector whose components are minus the right-hand sides of the equations and call it b.*)
{b, m} = CoefficientArrays[eq, x];

(*Call the number of rows of m rowsm. Call the number of columns of m colsm.*)
{rowsm, colsm} = Dimensions[m];

(* Augment m with IdentityMatrix[rowsm] on the right and row reduce. Call the resulting reduced augmented coefficient matrix racm. Call the first colsm columns of racm reducedm. Call the remaining columns t. The matrix t is such that m.t=reducedm and applying the same row operations to b yields t.b. Call this vector reducedb.*)

racm = RowReduce[Transpose[Join[Transpose[m], IdentityMatrix[rowsm]]]];
t = racm[[All, colsm + 1 ;; colsm + rowsm]];
reducedm = racm[[All, 1 ;; colsm]];
reducedb = t . b;

(*check that all the entries of t are integers*)
(*MatrixQ[t,IntegerQ]*)

(*Search reducedb for nonzero entries below the last nonzero row of reducedm, which is row MatrixRank[m]. Export the resulting list to numbers.txt. The gcd of these numbers is a maximum modulus for which the equations have a solution.*) 

Export["numbers10.txt", Cases[reducedb[[MatrixRank[m] + 1 ;; rowsm]], Except[0]]];

(*numbers10.txt is empty, so the system of equations has integer solutions. Call the first 22 rows of reducedm salientm0, and call the first 22 entries of reducedb salientb. Remove the columns of 0s from salientm0 and call the resulting matrix salientm. Output a file salientm10.txt containing salientm. Output a file salientb10.txt containing salientb.*)

salientm0 = reducedm[[1 ;; 22, All]];
salientb = reducedb[[1 ;; 22]];
salientm = Transpose[Cases[Transpose[salientm0], Except[ConstantArray[0, 22]]]];
Export["salientm10.txt", {salientm}];
Export["salientb10.txt",salientb];

(*salientm10 is a 22-by-22 identity matrix on the left side, with the matrix of coefficients for the salient solution set on the right side. The coefficient matrix has rank 20 as shown below:*)

{rowssalientm,colssalientm}=Dimensions[salientm];
MatrixRank[salientm[[All,23;;colssalientm]]]

(*The only output in the notebook should be 20, meaning the salient set is a 20-dimensional affine subspace of Z^22.*)