(*Description. This is the list of commands to produce three outputs: numbers7.txt, salientm.txt and salientb.txt. The file numbers7.txt has a list of numbers whose gcd is the modulus for which each salient sequence described by the other files comes from a valid choice of grading. In this case, that modulus is 0, so the grading system has integer solutions. The collection of salient sequences are of the form -salientm7.x-salientb7, where x ranges over Z^22, salientm7 is the matrix in salientm7.txt, and salientb7 is the vector in salientb7.txt.*)

(*Instructions. In Linux, feed this file to Mathematica with Equations7.txt in Mathematica's default directory using, say, "math < 7.m 2>&1 | tee 7.log". In Windows,open a notebook,type the command Directory[] and hit Shift+Enter.Put Equations7.txt into the folder outputted by Mathematica.Next,copy/paste this entire file into the notebook,and hit Shift+Enter. For Equations7.txt, it took a Surface Book 2 with i7 processor about 20 minutes to complete. To run your own equations, change the Import command to bring in Equations.txt instead of Equations7.txt, and change the export commands below to output whatever named files you like, other than numbers7.txt, and salientm7.txt, salientb7.txt.*)

(*Here begins the annotated sequence of Mathematica commands.*)

(*This is a command to delete all previously defined variables.It is commented out,You may want to un-comment it by deleting the (**).The semicolon tells Mathematica to not print any output.*)(*ClearAll["Global`*"];*)(*Import the list of equations for $E(1)_{7_6}$.Call the resulting string eqstring.*)
eqstring=Import["Equations7.txt"];

(*Interpret eqstring as a list of equations instead of just a text string.Call the resulting list of equations eq*)
eq=ToExpression[eqstring];

(*List all of the variables in eq in the order in which they appear,including repeats.Call the resulting list of strings xstring.*)
xstring=StringCases[eqstring,RegularExpression["c[[0-9]+,[0-9]+,(h|v)]"]];

(*Interpret the entries of the list xstring as variables instead of just text strings.Call this list of variables x1.*)
x1=ToExpression[xstring];

(*Call the list of salient chords s.*)
s={c[1072,332,v],c[1056,316,v],c[1040,302,v],c[1024,286,v],c[1008,272,v],c[992,256,v],c[976,242,v],c[960,226,v],c[944,212,v],c[928,196,v],c[912,182,v],c[896,166,v],c[880,152,v],c[864,136,v],c[848,122,v],c[832,106,v],c[816,92,v],c[800,76,v],c[784,62,v],c[766,46,v],c[766,338,h],c[1090,338,v]};

(*Insert the entries of s at the beginning of x1.Call this new list of variables x2.*)
x2=Join[s,x1];

(*Delete the duplicates in x2.Call the resulting list of variables,which begins with the list of salient chords,x.*)
x=DeleteDuplicates[x2];

(*Create the coefficient array for the equations' left-hand sides and call it m.Create the row vector whose components are minus the right-hand sides of the equations and call it b.*)
{b,m}=CoefficientArrays[eq,x];

(*Call the number of rows of m rowsm.Call the number of columns of m colsm.*)
{rowsm,colsm}=Dimensions[m];

(*Augment m with IdentityMatrix[rowsm] on the right and row reduce.Call the resulting reduced augmented coefficient matrix racm.Call the first colsm columns of racm reducedm.Call the remaining columns t.The matrix t is such that m.t=reducedm and applying the same row operations to b yields t.b.Call this vector reducedb.*)
racm=RowReduce[Transpose[Join[Transpose[m],IdentityMatrix[rowsm]]]];
t=racm[[All,colsm+1;;colsm+rowsm]];
reducedm=racm[[All,1;;colsm]];
reducedb=t.b;

(*check that all the entries of t are integers*)
(*MatrixQ[t,IntegerQ]*)

(*Search reducedb for nonzero entries below the last nonzero row of reducedm,which is row MatrixRank[m].Export the resulting list to numbers.txt.The gcd of these numbers is a maximum modulus for which the equations have a solution.*)

Export["numbers7.txt",Cases[reducedb[[MatrixRank[m]+1;;rowsm]],Except[0]]];

(*Numbers7.txt is empty, so the system of equations has integer solutions. Call the first 22 rows of reducedm salientm0,and call the first 22 entries of reducedb salientb.Remove the columns of 0s from salientm0 and call the resulting matrix salientm.Output a txt file containing salientm.*)

salientm0=reducedm[[1;;22,All]];
salientb=reducedb[[1;;22]];
salientm=Transpose[Cases[Transpose[salientm0],Except[ConstantArray[0,22]]]];
Export["salientm7.txt",{salientm}];
Export["salientb7.txt",salientb];

(*salientm7 is a 22-by-22 identity matrix on the left side, with the matrix of coefficients for the salient solution set on the right side. The coefficient matrix has rank 21 as shown below:*)

{rowssalientm,colssalientm}=Dimensions[salientm];
MatrixRank[salientm[[All,23;;colssalientm]]]
(*Output is expected to be "Out[21]= 21", meaning the salient set is a 21-dimensional affine subspace of Z^22.*)